/*
   This file is part of the index package,
   Copyright (C) 1997 Harold Tay, haroldtay@pacific.net.sg

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA
 */
/* Dump an index.
 */

#include <stdio.h>
#include <string.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <errno.h>
#include "index.h"
#include "codec.h"

char * progname;

void
usage(void)
{
  fprintf(stderr, "Usage: %s [-c] [<indexfile]\n", progname);
  exit(1);
}

struct meta meta;

int
main(int argc, char ** argv)
{
  char * map;
  char * locations;
  char * words;
  struct stat st;
  int countonly = 0;
  char * cp;
  int i;

  progname = ((cp = strrchr(argv[0], '/'))?cp + 1:argv[0]);

  for(i = 1; i < argc; i++)
    if( '-' == *argv[i] )
      if( 'c' == argv[i][1] )countonly = 1;
      else usage();
    else
      usage();

  if( -1 == fstat(0, &st) ){
    fprintf(stderr, "Can't stat 0: %s\n", strerror(errno));
    exit(1);
  }

  map = mmap(0, st.st_size, PROT_READ, MAP_FILE|MAP_PRIVATE, 0, 0);
  if( (int)map <= 0 ){
    fprintf(stderr, "Can't map 0: %s\n", strerror(errno));
    exit(1);
  }
  memcpy(&meta, map, sizeof(meta));
  if( strcmp(meta.magic, MAGIC) ){
    fprintf(stderr, "Wrong magic\n");
    exit(2);
  }
  locations = map + sizeof(meta);
  words = map + sizeof(meta) + meta.locations_size;

  while( words + 1 < map + st.st_size ){
    char * w = words;
    int wlen;
    int arlen;

    wlen = strlen(words);
    words += (wlen + 1);

    if( countonly ){
      words += codec_arraylen(words, &arlen);
      fprintf(stdout, "%d %s\n", arlen, w);
    }else{
      int * ary;
      int i;
      words += codec_dec(words, &ary, &arlen);
      fprintf(stdout, "%s\n", w);
      for(i = 0; i < arlen; i++){
        char * loc = locations + ary[i];
        if( loc < locations || loc >= w ){
          fflush(stdout);
          fprintf(stderr, "Bad location for word \"%s\"\n", w);
          exit(3);
        }
        fprintf(stdout, "\t%s\n", loc);
      }
    }
  }
  munmap(map, st.st_size);
  exit(0);
}

